/*
 * Copyright (C) 2007-2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gprintf.h>
#include <stdlib.h>

#include "libappstarter/as.h"

#define PLUGIN "mirage"

enum {
    FS = 0,
    RND,
};

typedef struct _SlPlugin {
    int fullscreen;
    gchar *path;
} SlPlugin;

SlPlugin *plugin;
gchar *cmd = NULL;
gchar *conffile = NULL;

static void
read_config ()
{
    XfceRc *rc;
    rc = as_rc_open(conffile, TRUE);
    if (!rc)
        return;
    xfce_rc_set_group (rc, PLUGIN);
    plugin->fullscreen = xfce_rc_read_int_entry (rc, "fullscreen", 1);
    plugin->path = g_strdup (xfce_rc_read_entry (
        rc, "path", g_getenv ("HOME")));
    xfce_rc_close (rc);
}

static void
write_config ()
{
    XfceRc *rc;
    rc = as_rc_open(conffile, FALSE);
    if (!rc)
        return;
    xfce_rc_set_group (rc, PLUGIN);
    xfce_rc_write_int_entry (rc, "fullscreen", plugin->fullscreen);
    xfce_rc_write_entry (rc, "path", plugin->path);
    xfce_rc_close (rc);
}

/* callbacks */
static void
entry_changed (GtkWidget *fe, gpointer data)
{
    g_free (plugin->path);
    plugin->path = g_strdup (
        as_file_entry_get_filename (AS_FILE_ENTRY (fe)));
    write_config();
}

static void
chk_toggled (GtkWidget *btn, gpointer data)
{
    int id;
    id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (btn), "as-id"));

    switch (id) {
        case FS:
            plugin->fullscreen = (gtk_toggle_button_get_active (
                GTK_TOGGLE_BUTTON (btn))) ? 1 : 0;
            break;
    }
    write_config ();
}

GtkWidget *
get_config_widget(gchar *file)
{
    GtkWidget *vbox, *fs_chk;
    GtkWidget *lbl, *hbox;
    GtkWidget *fe;

    conffile = g_strdup (file);
    plugin = g_new0 (SlPlugin, 1);
    plugin->path = NULL;
    read_config();

    vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
    hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
    gtk_widget_show (hbox);
    lbl = gtk_label_new (_("Select a folder: "));
    gtk_box_pack_start (GTK_BOX (hbox), lbl, FALSE, FALSE, 2);
    gtk_widget_show (lbl);
    fe = as_file_entry_new (_("Select a folder"),
                            GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
                            plugin->path);
    gtk_box_pack_start (GTK_BOX (hbox), fe, TRUE, TRUE, 2);

    fs_chk = gtk_check_button_new_with_label (_("Play in full screen"));
    g_object_set_data (G_OBJECT (fs_chk), "as-id", GUINT_TO_POINTER (FS));
    gtk_box_pack_start (GTK_BOX (vbox), fs_chk, FALSE, FALSE, 2);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fs_chk),
        (plugin->fullscreen) ? TRUE : FALSE);
    gtk_widget_show (fs_chk);

    lbl = gtk_label_new ("");
    gtk_label_set_markup (GTK_LABEL (lbl),
      _("<span size=\"small\"><span style=\"italic\">"
        "Note: </span>"
        "You can set the interval to wait between "
        "each image in the Mirage preferences.</span>"));
    gtk_misc_set_alignment (GTK_MISC (lbl), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (vbox), lbl, FALSE, FALSE, 2);
    gtk_widget_show (lbl);

    g_signal_connect (G_OBJECT (fe), "changed",
        G_CALLBACK (entry_changed), NULL);
    g_signal_connect (G_OBJECT (fs_chk), "toggled",
        G_CALLBACK (chk_toggled), NULL);

    return vbox;
}

gchar *
get_cmd()
{
    g_free (cmd);
    cmd = g_strconcat ("mirage -s ",
                       (plugin->fullscreen) ? "-f " : "",
                       plugin->path,
                       NULL);
    return cmd;
}

void
free_all()
{
    g_free (cmd);
    g_free (plugin->path);
    g_free (plugin);
    g_free (conffile);
}
